from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list, subtract=False):
    """Return type is always dict, the argument is a remnant of an undecided phase of the project."""
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}


    sent_id = conllu_item['sent_id']

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i


    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []
    head_exist = []

    for a in nanalyses:
        
        # this is what the PronTags file said, probably not exactly as intended

        if a[3] == 'PRON' and 'Person=' in a[5] and a[7] == 'det':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    zero_marked.append(a)
        

        elif a[3] == 'PRON' and 'Person=' in a[5] and a[7] == 'nmod':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(a)



        elif a[3] == 'PART' and 'Case=Gen' in a[5] and a[7] == 'case:dec':
            part_index = int(a[0])
            dep_i = find_head_index(a)
            if dep_i >= 0:
                dep = analyses[dep_i]
                if dep[3] in {'NOUN', 'PROPN'} and dep[7] == 'det':
                    head_i = find_head_index(dep)
                    if head_i >= 0:
                        if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                            dep_marked.append(dep + [part_index] if subtract else dep)
                            
                            for wd in canalyses.copy():
                            # find conjs for this "condition"
                            # Differences:
                            # assume dep is found:
                            # (start from testing the dependent, replacing dep with wd and also checking its head is dep (firstconj))
                            # dep -> wd
                            # firstconj_i
                            # replace dep(would be wd)[7] == 'nmod' with wd[6] == dep[0]
                            # append a manipulated version of the word (to_append) to calculate the dependency length correctly
                            # remove word from canalyses
                                if wd[3] in {'NOUN', 'PROPN'} and wd[6] == dep[0]:
                                    firstconj = dep
                                    head_i = find_head_index(firstconj)
                                    if head_i >= 0:
                                        head = analyses[head_i] # this line, as well as the replacement analyses[head_i][3] -> head[3] on the next, were added manually after tagchart_script failed due to UnboundLocalError: local variable 'head' referenced before assignment
                                        if head[3] in {'NOUN', 'PROPN'}:
                                            head_position = head[0]
                                            to_append = wd[:6] + [head_position] + wd[7:]
                                            dep_marked.append(to_append + [part_index] if subtract else to_append)
                                            
                                            canalyses.remove(wd) # just for sure

    # almost copy-paste code.
    # Differences:
    # -firstconj lines
    # -canalyses instead of nanalyses
    # -a[7] replaced with firstconj[7]
    # -appending an edited version of the dependent
    # -skipping adpositions and others whose checking processes don't start with the dependent

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]

    for a in canalyses:
        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent
        

        
        # this is what the PronTags file said, probably not exactly as intended

        if a[3] == 'PRON' and 'Person=' in a[5] and firstconj[7] == 'det':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    zero_marked.append(appendable(firstconj, a))
        

        elif a[3] == 'PRON' and 'Person=' in a[5] and firstconj[7] == 'nmod':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(appendable(firstconj, a))


    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer


test = r'''# sent_id = dev-s2
# text = 大多數的加長型禮車則是租車公司的財產.
1	大	大	ADV	RB	_	2	advmod	_	SpaceAfter=No
2	多數	多數	ADJ	JJ	_	6	det	_	SpaceAfter=No
3	的	的	PART	DEC	Case=Gen	2	case:dec	_	SpaceAfter=No
4	加長	加長	VERB	VV	_	5	case:suff	_	SpaceAfter=No
5	型	型	PART	SFN	_	6	nmod	_	SpaceAfter=No
6	禮車	禮車	NOUN	NN	_	11	nsubj	_	SpaceAfter=No
7	則是	則是	AUX	VC	_	11	cop	_	SpaceAfter=No
8	租車	租車	NOUN	NN	_	9	nmod	_	SpaceAfter=No
9	公司	公司	NOUN	NN	_	11	det	_	SpaceAfter=No
10	的	的	PART	DEC	Case=Gen	9	case:dec	_	SpaceAfter=No
11	財產	財產	NOUN	NN	_	0	root	_	SpaceAfter=No
12	.	.	PUNCT	.	_	11	punct	_	SpaceAfter=No'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item])
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=list):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=list):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)

