from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list, subtract=False):
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}


    sent_id = conllu_item['sent_id']
    

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i

    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []
    head_exist = []

    for a in nanalyses: # a could be the dependent or the adposition (equivalent of 'of'),
        #which is the reason that the initial variable name is nothing more descriptive than that.
        #The name of the head variable ("head" or "dep") is supposed to be more descriptive:
        #if it's "head", then "a" is a dependent, if it's "dep", then "a" is the adposition.
        
        # pronouns

        if a[3] == 'PRON' and 'PronType=Prs' in a[5] and a[7] == 'nmod:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(a)

        # nouns

        if a[2] == 'van' and a[7] == 'case':
            part_index = int(a[0])
            dep_i = find_head_index(a)
            if dep_i >= 0:
                dep = analyses[dep_i]
                if dep[3] in {'NOUN', 'PROPN'} and dep[7] == 'nmod':
                    head_i = find_head_index(dep)
                    if head_i >= 0:
                        head = analyses[head_i]
                        if head[3] in {'NOUN', 'PROPN'}:
                            dep_marked.append(dep + [part_index] if subtract else dep)
                            
                            for wd in canalyses.copy():
                            # find conjs for this "condition"
                            # Differences:
                            # assume dep is found:
                            # (start from testing the dependent, replacing dep with wd and also checking its head is dep (firstconj))
                            # dep -> wd
                            # firstconj_i
                            # replace dep(would be wd)[7] == 'nmod' with wd[6] == dep[0]
                            # append a manipulated version of the word (to_append) to calculate the dependency length correctly
                            # remove word from canalyses
                                if wd[3] in {'NOUN', 'PROPN'} and wd[6] == dep[0]:
                                    firstconj = dep
                                    head_i = find_head_index(firstconj)
                                    if head_i >= 0:
                                        head = analyses[head_i]
                                        if head[3] in {'NOUN', 'PROPN'}:
                                            head_position = head[0]
                                            to_append = wd[:6] + [head_position] + wd[7:]
                                            dep_marked.append(to_append + [part_index] if subtract else to_append)
                                            
                                            canalyses.remove(wd) # just for sure
        

        if a[1].lower() != 'de' and a[2] == 'de' and a[7] == 'det':
            part_index = int(a[0])
            dep_i = find_head_index(a) # if word form is de and lemma is de, it's not 'of'; word form in these cases is usually "der"
            if dep_i >= 0:
                dep = analyses[dep_i]
                if dep[3] in {'NOUN', 'PROPN'} and dep[7] == 'nmod':
                    head_i = find_head_index(dep)
                    if head_i >= 0:
                        head = analyses[head_i]
                        if head[3] in {'NOUN', 'PROPN'} and [j for j in analyses if ((j[2] == 'van') or (j[3] == 'ADP' and j[7] == 'case')) and j[6] == dep[0]] == []:
                            dep_marked.append(dep + [part_index] if subtract else dep)
                            
                            for wd in canalyses.copy():
                            # find conjs for this "condition"
                            # Differences:
                            # assume dep is found:
                            # (start from testing the dependent, replacing dep with wd and also checking its head is dep (firstconj))
                            # dep -> wd
                            # firstconj_i
                            # replace dep(would be wd)[7] == 'nmod' with wd[6] == dep[0]
                            # append a manipulated version of the word (to_append) to calculate the dependency length correctly
                            # remove word from canalyses
                                if wd[3] in {'NOUN', 'PROPN'} and wd[6] == dep[0]:
                                    firstconj = dep
                                    head_i = find_head_index(firstconj)
                                    if head_i >= 0:
                                        head = analyses[head_i]
                                        if head[3] in {'NOUN', 'PROPN'} and [j for j in analyses if ((j[2] == 'van') or (j[3] == 'ADP' and j[7] == 'case')) and j[6] == dep[0]] == []:
                                            head_position = head[0]
                                            to_append = wd[:6] + [head_position] + wd[7:]
                                            dep_marked.append(to_append + [part_index] if subtract else to_append)
                                            
                                            canalyses.remove(wd) # just for sure
    
    # almost copy-paste code.
    # Differences:
    # -firstconj lines
    # -canalyses instead of nanalyses
    # -a[7] replaced with firstconj[7]
    # -appending an edited version of the dependent
    # -skipping adpositions and others whose checking processes don't start with the dependent

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]

    for a in canalyses:
        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent
        

        if a[3] == 'PRON' and 'PronType=Prs' in a[5] and firstconj[7] == 'nmod:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(appendable(firstconj, a))


    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer


test = r'''# sent_id = LassySmall5_WR-P-P-H-0000000012_WR-P-P-H-0000000012.p.10.s.2.xml
# text = Egypte had, conform de werkelijke krachtsverhoudingen, al in de tweede minuut van de tweede helft toegeslagen.
1	Egypte	Egypte	PROPN	N|eigen|ev|basis|onz|stan	Gender=Neut|Number=Sing	18	nsubj	18:nsubj	_
2	had	hebben	AUX	WW|pv|verl|ev	Number=Sing|Tense=Past|VerbForm=Fin	18	aux	18:aux	SpaceAfter=No
3	,	,	PUNCT	LET	_	2	punct	2:punct	_
4	conform	conform	ADJ	ADJ|vrij|basis|zonder	Degree=Pos	7	case	7:case	_
5	de	de	DET	LID|bep|stan|rest	Definite=Def	7	det	7:det	_
6	werkelijke	werkelijk	ADJ	ADJ|prenom|basis|met-e|stan	Degree=Pos	7	amod	7:amod	_
7	krachtsverhoudingen	krachtsverhouding	NOUN	N|soort|mv|basis	Number=Plur	18	obl	18:obl:conform	SpaceAfter=No
8	,	,	PUNCT	LET	_	7	punct	7:punct	_
9	al	al	ADV	BW	_	18	advmod	18:advmod	_
10	in	in	ADP	VZ|init	_	13	case	13:case	_
11	de	de	DET	LID|bep|stan|rest	Definite=Def	13	det	13:det	_
12	tweede	twee	ADJ	TW|rang|prenom|stan	_	13	amod	13:amod	_
13	minuut	minuut	NOUN	N|soort|ev|basis|zijd|stan	Gender=Com|Number=Sing	18	obl	18:obl:in	_
14	van	van	ADP	VZ|init	_	17	case	17:case	_
15	de	de	DET	LID|bep|stan|rest	Definite=Def	17	det	17:det	_
16	tweede	twee	ADJ	TW|rang|prenom|stan	_	17	amod	17:amod	_
17	helft	helft	NOUN	N|soort|ev|basis|zijd|stan	Gender=Com|Number=Sing	13	nmod	13:nmod:van	_
18	toegeslagen	toeslaan	VERB	WW|vd|vrij|zonder	VerbForm=Part	0	root	0:root	SpaceAfter=No
19	.	.	PUNCT	LET	_	18	punct	18:punct	_
'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item])
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)
