from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list, subtract=True):
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}

    sent_id = conllu_item['sent_id']
    

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i

    def find_chead_index(analysis):
        #if analysis[7] != conj: # only used for canalyses, this should be checked already
            #print(analysis, analysis[7])
        #assert analysis[7] == conj # only used for canalyses, this should be checked already
        b = find_head_index(analysis)
        if analyses[b][7] == '0':
            return -1
        return find_head_index(analyses[b])

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]

    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []
    head_exist = []

    for a in nanalyses: # a could be the dependent or the adposition (equivalent of 'of'),
        #which is the reason that the initial variable name is nothing more descriptive than that.
        #The name of the head variable ("head" or "dep") is supposed to be more descriptive:
        #if it's "head", then "a" is a dependent, if it's "dep", then "a" is the adposition.
        # pronouns
        if a[3] == 'DET' and 'Poss=Yes' in a[5] and a[7] == 'det:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(a)

        elif a[3] == 'PRON' and 'Poss=Yes' in a[5] and a[7] == 'det:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(a)

        elif a[3] == 'DET' and 'PronType=Prs' in a[5] and a[7] == 'det:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(a)

        # nouns  
        elif a[2] == 'von' and a[3] == 'ADP' and a[7] == 'case':
            part_index = int(a[0])
            dep_i = find_head_index(a) # von structures
            if dep_i >= 0:
                dep = analyses[dep_i]
                if dep[3] in {'NOUN', 'PROPN'} and 'Case=Dat' in dep[5] and dep[7] == 'nmod':
                    head_i = find_head_index(dep) # changed from a to dep on August 23
                    if head_i >= 0:
                        head = analyses[head_i]
                        if head[3] in {'NOUN', 'PROPN'}:
                            dep_marked.append(dep + [part_index] if subtract else dep)

                            for wd in canalyses.copy():
                                # find conjs for this "condition"
                                # Differences:
                                # dep -> wd
                                # firstconj_i
                                # replace dep(would be wd)[7] == 'nmod' with wd[6] == dep[0]
                                # append a manipulated version of the word (to_append) to calculate the dependency length correctly
                                # remove word from canalyses
                                if wd[3] in {'NOUN', 'PROPN'} and 'Case=Dat' in wd[5] and wd[6] == dep[0]:
                                    firstconj_i = find_head_index(wd) # firstconj index
                                    head_i = find_head_index(analyses[firstconj_i])
                                    
                                    if head_i > 0:
                                        head = analyses[head_i]
                                        if head[3] in {'NOUN', 'PROPN'}:
                                            head_position = head[0]
                                            to_append = wd[:6] + [head_position] + wd[7:]
                                            dep_marked.append(to_append + [part_index] if subtract else to_append)

                                            canalyses.remove(wd) # just for sure

                                            #print('von conj', sent_id, to_append)

        
        else: # no "von" in here, so no subtracting?
            dep = a
            #dep = analyses[dep_i]
            if dep[3] in {'NOUN', 'PROPN'} and 'Case=Gen' in dep[5] and dep[7] == 'nmod':
                head_i = find_head_index(dep)
                if head_i >= 0:
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        for word in analyses:
                            if word[6] == dep[0] and word[3] == 'DET' and word[7] == 'det':
                                dep_marked.append(dep)
                                break
                        else:
                            if dep[1].lower() == dep[2].lower():
                                zero_marked.append(dep)
                            else:
                                dep_marked.append(dep)

                        for wd in canalyses:
                            # find conjs for this "condition"
                            # Differences:
                            # dep -> wd
                            # firstconj_i
                            # replace dep(would be wd)[7] == 'nmod' with wd[6] == dep[0]
                            # append a manipulated version of the word (to_append) to calculate the dependency length correctly
                            # remove word from canalyses
                            if wd[3] in {'NOUN', 'PROPN'} and 'Case=Gen' in wd[5] and wd[6] == dep[0]:
                                firstconj_i = find_head_index(wd) # firstconj index
                                head_i = find_head_index(analyses[firstconj_i])
                                
                                if head_i >= 0:
                                    head = analyses[head_i]
                                    if head[3] in {'NOUN', 'PROPN'}:
                                        head_position = head[0]
                                        to_append = wd[:6] + [head_position] + wd[7:]
                                        for word in analyses:
                                            if word[6] == dep[0] and word[3] == 'DET' and word[7] == 'det':
                                                dep_marked.append(to_append)
                                                #print('art gen conj', sent_id, to_append)
                                                break
                                        else:
                                            if word[1].lower() == word[2].lower():
                                                zero_marked.append(to_append)
                                            else:
                                                dep_marked.append(to_append)
                                            #print('clemma conj', sent_id, to_append)
                                        canalyses.remove(wd) # just for sure

                                        

                        

    # almost copy-paste code.
    # Differences:
    # -canalyses instead of nanalyses
    # -dep[7] and a[7] replaced with firstconj[7]
    # -appending an edited version of the dependent
    # -no checking of whether the adposition ("von" in the case of German) is present, this has been checked in the previous loop


    for a in canalyses:

        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent
        
        if a[3] == 'DET' and 'Poss=Yes' in a[5] and firstconj[7] == 'det:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    #print(sent_id)
                    dep_marked.append(appendable(firstconj, a))

        elif a[3] == 'PRON' and 'Poss=Yes' in a[5] and firstconj[7] == 'det:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    #print(sent_id)
                    dep_marked.append(appendable(firstconj, a))

        elif a[3] == 'DET' and 'PronType=Prs' in a[5] and firstconj[7] == 'det:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    #print(sent_id)
                    dep_marked.append(appendable(firstconj, a))

        if dep[3] in {'NOUN', 'PROPN'} and 'Case=Gen' in dep[5] and dep[7] == 'nmod':
            head_i = find_head_index(dep)
            if head_i >= 0:
                head = analyses[head_i]
                if head[3] in {'NOUN', 'PROPN'}:
                    #print(sent_id)
                    dep_marked.append(appendable(firstconj, dep))

    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer


test = r'''# sent_id = dev-s502
# text = Dieter Spöri, SPD-Wirtschaftsminister in Teufels großer Koalition Baden-Württembergs, findet, daß die SPD aus den Folgen des radikalen wirtschaftlichen Wandels, der Globalisierung von Arbeitsmarkt und Produktion noch nicht die entscheidenden Folgerungen gezogen hat.
1	Dieter	Dieter	PROPN	NE	Case=Nom|Gender=Masc|Number=Sing	2	dep	_	FixTigerDep=Yes
2	Spöri	Spöri	PROPN	NE	Case=Nom|Gender=Masc|Number=Sing	15	nsubj	_	SpaceAfter=No
3	,	,	PUNCT	$,	_	6	punct	_	_
4	SPD	SPD	PROPN	NN	Case=Nom|Gender=Masc|Number=Sing	6	compound	_	SpaceAfter=No
5	-	-	PUNCT	$(	_	4	punct	_	SpaceAfter=No
6	Wirtschaftsminister	Wirtschaftsminister	NOUN	NN	Case=Nom|Gender=Masc|Number=Sing	2	appos	_	_
7	in	in	ADP	APPR	_	10	case	_	_
8	Teufels	Teufel	PROPN	NE	Case=Gen|Number=Sing	10	nmod	_	_
9	großer	groß	ADJ	ADJA	Case=Dat|Gender=Fem|Number=Sing	10	amod	_	_
10	Koalition	Koalition	NOUN	NN	Case=Dat|Gender=Fem|Number=Sing	4	nmod	_	_
11	Baden	Baden	PROPN	NE	Case=Gen|Gender=Neut|Number=Sing	10	nmod	_	SpaceAfter=No
12	-	-	PUNCT	$(	_	13	punct	_	SpaceAfter=No
13	Württembergs	Württemberg	PROPN	NE	Case=Gen|Gender=Neut|Number=Sing	11	flat	_	SpaceAfter=No
14	,	,	PUNCT	$,	_	2	punct	_	_
15	findet	finden	VERB	VVFIN	Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin	0	root	_	SpaceAfter=No
16	,	,	PUNCT	$,	_	39	punct	_	_
17	daß	daß	CCONJ	KOUS	_	39	cc	_	_
18	die	der	DET	ART	Case=Nom|Definite=Def|Gender=Fem|Number=Sing|PronType=Art	19	det	_	_
19	SPD	SPD	PROPN	NE	Case=Nom|Gender=Fem|Number=Sing	39	nsubj	_	_
20	aus	aus	ADP	APPR	_	22	case	_	_
21	den	der	DET	ART	Case=Dat|Definite=Def|Gender=Fem|Number=Plur|PronType=Art	22	det	_	_
22	Folgen	Folge	NOUN	NN	Case=Dat|Gender=Fem|Number=Plur	39	obl	_	_
23	des	der	DET	ART	Case=Gen|Definite=Def|Gender=Masc|Number=Sing|PronType=Art	26	det	_	_
24	radikalen	radikal	ADJ	ADJA	Case=Gen|Gender=Masc|Number=Sing	26	amod	_	_
25	wirtschaftlichen	wirtschaftlich	ADJ	ADJA	Case=Gen|Gender=Masc|Number=Sing	26	amod	_	_
26	Wandels	Wandel	NOUN	NN	Case=Gen|Gender=Masc|Number=Sing	22	nmod	_	SpaceAfter=No
27	,	,	PUNCT	$,	_	29	punct	_	_
28	der	der	DET	ART	Case=Gen|Definite=Def|Gender=Fem|Number=Sing|PronType=Art	29	det	_	_
29	Globalisierung	Globalisierung	NOUN	NN	Case=Gen|Gender=Fem|Number=Sing	26	conj	_	_
30	von	von	ADP	APPR	_	31	case	_	_
31	Arbeitsmarkt	Arbeitsmarkt	NOUN	NN	Case=Dat|Gender=Masc|Number=Sing	29	nmod	_	_
32	und	und	CCONJ	KON	_	33	cc	_	_
33	Produktion	Produktion	NOUN	NN	Case=Dat|Gender=Fem|Number=Sing	31	conj	_	_
34	noch	noch	ADV	ADV	_	35	advmod	_	_
35	nicht	nicht	PART	PTKNEG	Polarity=Neg	39	advmod	_	_
36	die	der	DET	ART	Case=Acc|Definite=Def|Gender=Fem|Number=Plur|PronType=Art	38	det	_	_
37	entscheidenden	entscheidend	ADJ	ADJA	Case=Acc|Gender=Fem|Number=Plur	38	amod	_	_
38	Folgerungen	Folgerung	NOUN	NN	Case=Acc|Gender=Fem|Number=Plur	39	obj	_	_
39	gezogen	ziehen	VERB	VVPP	VerbForm=Part	15	ccomp	_	_
40	hat	haben	AUX	VAFIN	Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin	39	aux	_	SpaceAfter=No
41	.	.	PUNCT	$.	_	15	punct	_	_'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item])
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)
