from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list):
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}


    sent_id = conllu_item['sent_id']

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i


    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []
    head_exist = []

    for a in nanalyses: # It seems like there is nothing but dependent-marking in the material
        #print('a in analyses, a =', a)
        # here a is dependent

        if a[3] == 'PRON' and 'PronType=Prs' in a[5] and a[7] == 'nmod:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(a)
        # a is particle

        elif a[3] == 'ADP' and a[7] == 'case:gen': # lemma usually 'של', but not relevant for this algorithm
            dep_i = find_head_index(a)
            if dep_i >= 0:
                dep = analyses[dep_i]
                if dep[3] in {'NOUN', 'PROPN'} and dep[7] == 'nmod:poss':
                    head_i = find_head_index(dep)
                    if head_i >= 0:
                        if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                            dep_marked.append(dep)
                            
                            for wd in canalyses.copy():
                            # find conjs for this "condition"
                            # Differences:
                            # assume dep is found:
                            # (start from testing the dependent, replacing dep with wd and also checking its head is dep (firstconj))
                            # dep -> wd
                            # firstconj_i
                            # replace dep(would be wd)[7] == 'nmod' with wd[6] == dep[0]
                            # append a manipulated version of the word (to_append) to calculate the dependency length correctly
                            # remove word from canalyses
                                if wd[3] in {'NOUN', 'PROPN'} and wd[6] == dep[0]:
                                    firstconj = dep
                                    head_i = find_head_index(firstconj)
                                    if head_i >= 0:
                                        head = analyses[head_i] # added manually to this otherwise generated piece of code due to error in command: head_position = head[0]
                                        if head[3] in {'NOUN', 'PROPN'}: # and this also changed from analyses[head_i][3] to head[3]
                                            head_position = head[0]
                                            to_append = wd[:6] + [head_position] + wd[7:]
                                            dep_marked.append(to_append)
                                            
                                            canalyses.remove(wd) # just for sure

    # almost copy-paste code.
    # Differences:
    # -firstconj lines
    # -canalyses instead of nanalyses
    # -a[7] replaced with firstconj[7]
    # -appending an edited version of the dependent
    # -skipping adpositions and others whose checking processes don't start with the dependent

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]

    for a in canalyses:
        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent
        

        #print('a in analyses, a =', a)
        # here a is dependent

        if a[3] == 'PRON' and 'PronType=Prs' in a[5] and firstconj[7] == 'nmod:poss':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(appendable(firstconj, a))

    
    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer



test = r'''# sent_id = 3465
# text = אולפני ההסרטה של מ-ג-מ, שנפתחו לקהל ב9891 סמוך לפארקים האחרים בשטח של דיסני באורלנדו, כבר מדיפים ריח של נפטלין, במונחים אמריקאיים.
1	אולפני	אולפן	NOUN	NOUN	Definite=Cons|Gender=Masc|Number=Plur	33	nsubj	_	_
2-3	ההסרטה	_	_	_	_	_	_	_	_
2	ה	ה	DET	DET	PronType=Art	3	det:def	_	_
3	הסרטה	הסרטה	NOUN	NOUN	Gender=Fem|Number=Sing	1	compound:smixut	_	_
4	של	של	ADP	ADP	Case=Gen	5	case:gen	_	_
5	מ	מ	PROPN	PROPN	_	1	nmod	_	SpaceAfter=No
6	-	-	PUNCT	PUNCT	_	5	flat:name	_	SpaceAfter=No
7	ג	ג	PROPN	PROPN	_	5	flat:name	_	SpaceAfter=No
8	-	-	PUNCT	PUNCT	_	5	flat:name	_	SpaceAfter=No
9	מ	מ	PROPN	PROPN	_	5	flat:name	_	SpaceAfter=No
10	,	,	PUNCT	PUNCT	_	1	punct	_	_
11-12	שנפתחו	_	_	_	_	_	_	_	_
11	ש	ש	SCONJ	SCONJ	_	12	mark	_	_
12	נפתחו	נפתח	VERB	VERB	Gender=Fem,Masc|HebBinyan=NIFAL|Number=Plur|Person=3|Tense=Past|Voice=Mid	1	acl:relcl	_	_
13-15	לקהל	_	_	_	_	_	_	_	_
13	ל	ל	ADP	ADP	_	15	case	_	_
14	ה_	ה	DET	DET	PronType=Art	15	det:def	_	_
15	קהל	קהל	NOUN	NOUN	Gender=Masc|Number=Sing	12	obl	_	_
16-17	ב9891	_	_	_	_	_	_	_	_
16	ב	ב	ADP	ADP	_	17	case	_	_
17	9891	9891	NUM	NUM	_	12	obl	_	_
18	סמוך	_	ADV	ADV	_	12	advmod	_	_
19-21	לפארקים	_	_	_	_	_	_	_	_
19	ל	ל	ADP	ADP	_	21	case	_	_
20	ה_	ה	DET	DET	PronType=Art	21	det:def	_	_
21	פארקים	פארק	NOUN	NOUN	Gender=Masc|Number=Plur	18	fixed	_	_
22-23	האחרים	_	_	_	_	_	_	_	_
22	ה	ה	DET	DET	PronType=Art	23	det:def	_	_
23	אחרים	אחר	ADJ	ADJ	Gender=Masc|Number=Plur	21	amod	_	_
24-26	בשטח	_	_	_	_	_	_	_	_
24	ב	ב	ADP	ADP	_	26	case	_	_
25	ה_	ה	DET	DET	PronType=Art	26	det:def	_	_
26	שטח	שטח	NOUN	NOUN	Gender=Masc|Number=Sing	12	obl	_	_
27	של	של	ADP	ADP	Case=Gen	28	case:gen	_	_
28	דיסני	דיסני	PROPN	PROPN	_	26	nmod	_	_
29-30	באורלנדו	_	_	_	_	_	_	_	SpaceAfter=No
29	ב	ב	ADP	ADP	_	30	case	_	_
30	אורלנדו	אורלנדו	PROPN	PROPN	_	26	nmod	_	_
31	,	,	PUNCT	PUNCT	_	33	punct	_	_
32	כבר	כבר	ADV	ADV	_	33	advmod	_	_
33	מדיפים	הדיף	VERB	VERB	Gender=Masc|HebBinyan=HIFIL|Number=Plur|Person=1,2,3|VerbForm=Part|Voice=Act	0	root	_	_
34	ריח	ריח	NOUN	NOUN	Gender=Masc|Number=Sing	33	obj	_	_
35	של	של	ADP	ADP	Case=Gen	36	case:gen	_	_
36	נפטלין	נפטלין	NOUN	NOUN	Gender=Masc|Number=Sing	34	nmod:poss	_	SpaceAfter=No
37	,	,	PUNCT	PUNCT	_	39	punct	_	_
38-39	במונחים	_	_	_	_	_	_	_	_
38	ב	ב	ADP	ADP	_	39	case	_	_
39	מונחים	מונח	NOUN	NOUN	Gender=Masc|Number=Plur	33	obl	_	_
40	אמריקאיים	אמריקני	ADJ	ADJ	Gender=Masc|Number=Plur	39	amod	_	SpaceAfter=No
41	.	.	PUNCT	PUNCT	_	33	punct	_	_'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item])
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)
        
if __name__ == '__main__':
    exec(r'''import os

while 'UDtrack' in os.listdir():
    os.chdir('UDtrack')

folders = [w for w in os.listdir() if LANGUAGE in w and '.' not in w]

rootfolder = os.getcwd()

answer = ''

for folder in folders:
    os.chdir(folder)
    files = os.listdir()
    if folder[3:] + '_THIS.conllu' in files:
        file = folder[3:] + '_THIS.conllu'
        # A final modified file should be have a name that ends with
        # _THIS.conllu
        # e.g. UD_Afrikaans-AfriBooms_THIS.conllu
    else:
        file = folder[3:] + '.conllu'
        # If such a file doesn't exist, just use the original which is named
        # folder without UD_ + '.conllu'

    answer += avaa(file)

    os.chdir(rootfolder)

answer = answer.strip()

g = answer.split('\n\n') # merkkijonot
''')
    g = [w for w in g if w.strip()]
    h = [to_ordered_dict(w) for w in g]
    zero = []
    for sent in h:
        if possessive(sent, dict)['zero_marked']:
            zero.append(sent)
    

