from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list, subtract=False):
    """Will always return a dict, the returntype argument is needless, but a remnant of a previous phase of the project"""
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}


    sent_id = conllu_item['sent_id']

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i
    
    def find_chead_index(analysis):
        b = find_head_index(analysis)
        if analyses[b][6] == '0':
            return -1
        return find_head_index(analyses[b])

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]

    def has_adpositions(a): # True if there is an adposition referring to a
        for j in analyses:
            if j[3] == 'ADP' and j[7] == 'case' and j[6] == a[0]:
                return True
        return False

    dep_head_exist = [] # items are dependents
    head_exist = [] # items are heads

    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []

    for a in nanalyses:
        # a is DEPENDENT
        head_i = find_head_index(a)
        if head_i >= 0: # head must exist            
            if a[2].endswith('의') and a[3] in {'PRON', 'NOUN', 'PROPN'} and a[7] == 'nmod' and analyses[head_i][3] in {'NOUN', 'PROPN'}:
                dep_marked.append(a) # Korean-Kaist

        # almost copy-paste code.
    # Differences:
    # -canalyses instead of nanalyses
    # -a[7] replaced with firstconj[7]
    # -appending an edited version of the dependent

    for a in canalyses:

        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent

        # a is DEPENDENT
        head_i = find_head_index(analyses[firstconj_i])
        
        if head_i >= 0: # head must exist
            if a[2].endswith('의') and a[3] in {'PRON', 'NOUN', 'PROPN'} and firstconj[7] == 'nmod' and analyses[head_i][3] in {'NOUN', 'PROPN'}:
                dep_marked.append(appendable(firstconj, a)) # Korean-Kaist
    
    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer

# Sentence is manipulated
test = r'''# sent_id = M2TA_069-s22
# text = 그 다리를 건너서 증포리 , 사근리 , 왕십리 , 마장리로 쳐들어왔습니다 .
1	그	그	DET	mmd	_	2	det	_	_
2	다리를	다리+를	NOUN	ncn+jco	_	3	obj	_	_
3	건너서	건너+서	SCONJ	pvg+ecs	_	11	ccomp	_	_
4	증포리의	증포리의	PROPN	nq	_	10	nmod	_	_
5	,	,	PUNCT	sp	_	4	punct	_	_
6	사근리의	사근리의	PROPN	nq	_	4	conj	_	_
7	,	,	PUNCT	sp	_	6	punct	_	_
8	왕십리의	왕십리의	PROPN	nq	_	4	conj	_	_
9	,	,	PUNCT	sp	_	8	punct	_	_
10	마장리로	마장리+로	NOUN	nq+jca	_	4	conj	_	_
11	쳐들어왔습니다	쳐들+어+오+았+습니다	AUX	pvg+ecx+px+ep+ef	_	0	root	_	_
12	.	.	PUNCT	sf	_	11	punct	_	_'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)
        
if __name__ == '__main__':
    exec(r'''import os

while 'UDtrack' in os.listdir():
    os.chdir('UDtrack')

folders = [w for w in os.listdir() if LANGUAGE in w and '.' not in w]

rootfolder = os.getcwd()

answer = ''

for folder in folders:
    os.chdir(folder)
    files = os.listdir()
    if folder[3:] + '_THIS.conllu' in files:
        file = folder[3:] + '_THIS.conllu'
        # A final modified file should be have a name that ends with
        # _THIS.conllu
        # e.g. UD_Afrikaans-AfriBooms_THIS.conllu
    else:
        file = folder[3:] + '.conllu'
        # If such a file doesn't exist, just use the original which is named
        # folder without UD_ + '.conllu'

    answer += avaa(file)

    os.chdir(rootfolder)

answer = answer.strip()

g = answer.split('\n\n') # merkkijonot
''')
    g = [w for w in g if w.strip()]
    h = [to_ordered_dict(w) for w in g]
    zero = []
    for sent in h:
        if possessive(sent, dict)['zero_marked']:
            zero.append(sent)
    

