from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

# This script makes heavy use of the language-specific syntactic function,
# which, in the case of this material, makes it relatively easy to deal with
# conj's.

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list, subtract=False):
    try:
        analyses = conllu_item['analyysit']
        zanalyses = sorted(analyses, key=lambda i:(':conj|' in i[8] or i[8].endswith(':conj')))
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}

    sent_id = conllu_item['sent_id']
    
    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i

    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []
    head_exist = []

    for a in zanalyses: # sorted so that conjs are last
        if a[3] == 'DET' and 'Poss=Yes' in a[5] and 'PronType=Prs' in a[5]:
            if a[7] == 'det':
                head_i = find_head_index(a)
                if head_i >= 0:
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        dep_marked.append(a)
            elif a[7] == 'conj':
                firstconj_i = find_head_index(a)
                if firstconj_i >= 0:
                    firstconj = analyses[firstconj_i] # the word that has the real syntactic function
                    chead_i = find_head_index(firstconj) # the actual head, the head of the entire NP with conjunctions
                    if chead_i >= 0:
                        chead = analyses[chead_i]
                        if chead[3] in {'NOUN', 'PROPN'} and firstconj in dep_marked+zero_marked+head_marked+double_marked:
                            to_append = a[:6] + [firstconj[6]] + a[7:]
                            dep_marked.append(to_append)
                            #print('pronoun conj in sent', sent_id)
        elif a[3] in {'NOUN', 'PROPN'}:
            synts = a[8].split('|')
            pairs = [j.split(':', 1) for j in synts]
            functions = [j[1] for j in pairs]
            if 'nmod:gen' in functions:
                head_i = find_head_index(a)
                if head_i >= 0:
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        if a[1].lower() == a[2].lower():
                            zero_marked.append(a)
                        else:
                            dep_marked.append(a)
            elif 'conj' in functions:
                firstconj_i = find_head_index(a)
                if firstconj_i >= 0:
                    firstconj = analyses[firstconj_i] # the word that has the real syntactic function
                    chead_i = find_head_index(firstconj) # the actual head, the head of the entire NP with conjunctions
                    if chead_i >= 0:
                        chead = analyses[chead_i]
                        if chead[3] in {'NOUN', 'PROPN'} and firstconj in dep_marked+zero_marked+head_marked+double_marked:
                            to_append = a[:6] + [firstconj[6]] + a[7:]
                            if a[1].lower() == a[2].lower():
                                zero_marked.append(to_append)
                            else:
                                dep_marked.append(to_append)
                            #print('noun conj in sent', sent_id)
        

    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer

test = r'''# sent_id = a-p999-p1s2
# text = Tos izgaismos sveču vai lukturīšu gaisma.
1	Tos	tas	PRON	pd3mpan	Case=Acc|Gender=Masc|Number=Plur|Person=3|PronType=Dem	2	obj	2:obj	LvtbNodeId=a-p999-p1s2w1
2	izgaismos	izgaismot	VERB	vmnift230an	Evident=Fh|Mood=Ind|Person=3|Polarity=Pos|Tense=Fut|VerbForm=Fin|Voice=Act	0	root	0:root	LvtbNodeId=a-p999-p1s2w2
3	sveču	svece	NOUN	ncfpg5	Case=Gen|Gender=Fem|Number=Plur	6	nmod	6:nmod:gen	LvtbNodeId=a-p999-p1s2w3
4	vai	vai	CCONJ	cc	_	5	cc	5:cc	LvtbNodeId=a-p999-p1s2w4
5	lukturīšu	lukturītis	NOUN	ncmpg2	Case=Gen|Gender=Masc|Number=Plur	3	conj	3:conj	LvtbNodeId=a-p999-p1s2w5
6	gaisma	gaisma	NOUN	ncfsn4	Case=Nom|Gender=Fem|Number=Sing	2	nsubj	2:nsubj	LvtbNodeId=a-p999-p1s2w6|SpaceAfter=No
7	.	.	PUNCT	zs	_	2	punct	2:punct	LvtbNodeId=a-p999-p1s2w7'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item])
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)


if __name__ == '__main__':
    from language_importer import *
    z = import_thing('Latvian')
    j = [possessive(to_ordered_dict(i)) for i in z]
    
