from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3].replace('HYPHEN', '-')

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list, subtract=False):
    """The return type is a dict, the "returntype" argument is a needless remnant of a previous phase of this project.
The point is to avoid crashing if an extra argument is given."""
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}


    sent_id = conllu_item['sent_id']

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i
    
    def find_chead_index(analysis):
        b = find_head_index(analysis)
        if analyses[b][6] == '0':
            return -1
        return find_head_index(analyses[b])

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]
    
    head_exist = []

    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []

    for a in nanalyses:

        #pronouns

        if a[3] == 'PRON' and 'Case=Gen' in a[5] and 'PronType=Prs' in a[5] and 'Person=' in a[5] and a[7] == 'nmod':
            head_i = find_head_index(a)
            if head_i >= 0:
                if not any(w[3] == 'ADP' and w[6] == a[0] for w in analyses): # dependent not allowed to be part of an adposition phrase
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        dep_marked.append(a)

        if a[3] == 'DET' and 'Case=Gen' in a[5] and 'Poss=Yes' in a[5] and 'PronType=Prs' in a[5] and 'Reflex=Yes' not in a[5] and a[7] == 'nmod':
            head_i = find_head_index(a)
            if head_i >= 0:
                if not any(w[3] == 'ADP' and w[6] == a[0] for w in analyses): # dependent not allowed to be part of an adposition phrase
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        dep_marked.append(a)

        #nouns

        if a[3] in {'NOUN', 'PROPN'} and 'Case=Gen' in a[5] and a[7] == 'nmod':
            head_i = find_head_index(a)
            if head_i >= 0:
                if not any(w[3] == 'ADP' and w[6] == a[0] for w in analyses): # dependent not allowed to be part of an adposition phrase
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        if a[1].lower() == a[2].lower(): # if the dependent's word form is the same as the lemma, then it's a zero marked example
                            zero_marked.append(a)
                        else:
                            dep_marked.append(a)

    # almost copy-paste code.
    # Differences:
    # -canalyses instead of nanalyses
    # -a[7] replaced with firstconj[7]
    # -appending an edited version of the dependent

    for a in canalyses:

        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent

        #pronouns

        if a[3] == 'PRON' and 'Case=Gen' in a[5] and 'PronType=Prs' in a[5] and 'Person=' in a[5] and firstconj[7] == 'nmod':
            head_i = find_chead_index(a)
            if head_i >= 0:
                if not any(w[3] == 'ADP' and w[6] == a[0] for w in analyses): # dependent not allowed to be part of an adposition phrase
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        dep_marked.append(appendable(firstconj, a))

        if a[3] == 'DET' and 'Case=Gen' in a[5] and 'Poss=Yes' in a[5] and 'PronType=Prs' in a[5] and 'Reflex=Yes' not in a[5] and firstconj[7] == 'nmod':
            head_i = find_chead_index(a)
            if head_i >= 0:
                if not any(w[3] == 'ADP' and w[6] == a[0] for w in analyses): # dependent not allowed to be part of an adposition phrase
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        dep_marked.append(appendable(firstconj, a))

        #nouns

        if a[3] in {'NOUN', 'PROPN'} and 'Case=Gen' in a[5] and firstconj[7] == 'nmod':
            head_i = find_chead_index(a)
            if head_i >= 0:
                if not any(w[3] == 'ADP' and w[6] == a[0] for w in analyses): # dependent not allowed to be part of an adposition phrase
                    head = analyses[head_i]
                    if head[3] in {'NOUN', 'PROPN'}:
                        if a[1].lower() == a[2].lower(): # if the dependent's word form is the same as the lemma, then it's a zero marked example
                            zero_marked.append(appendable(firstconj, a))
                        else:
                            dep_marked.append(appendable(firstconj, a))
        
    
    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer

##test = r'''# sent_id = dev-s25
### text = Obniża cenę do połowy, chociaż cena zależy też od stanu podręcznika.
### orig_file_sentence 001#25
##1	Obniża	obniżać	VERB	fin:sg:ter:imperf	Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin	0	root	_	_
##2	cenę	cena	NOUN	subst:sg:acc:f	Case=Acc|Gender=Fem|Number=Sing	1	obj	_	_
##3	do	do	ADP	prep:gen	AdpType=Prep|Case=Gen	4	case	_	_
##4	połowy	połowa	NOUN	subst:sg:gen:f	Case=Gen|Gender=Fem|Number=Sing	1	obl	_	SpaceAfter=No
##5	,	,	PUNCT	interp	_	8	punct	_	_
##6	chociaż	chociaż	SCONJ	comp	_	8	mark	_	_
##7	cena	cena	NOUN	subst:sg:nom:f	Case=Nom|Gender=Fem|Number=Sing	8	nsubj	_	_
##8	zależy	zależeć	VERB	fin:sg:ter:imperf	Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin	1	advcl	_	_
##9	też	też	PART	qub	_	8	advmod	_	_
##10	od	od	ADP	prep:gen:nwok	AdpType=Prep|Case=Gen|Variant=Short	11	case	_	_
##11	stanu	stan	NOUN	subst:sg:gen:m3	Animacy=Inan|Case=Gen|Gender=Masc|Number=Sing	8	obl:arg	_	_
##12	podręcznika	podręcznik	NOUN	subst:sg:gen:m3	Animacy=Inan|Case=Gen|Gender=Masc|Number=Sing	11	nmod	_	SpaceAfter=No
##13	.	.	PUNCT	interp	_	1	punct	_	_'''

test = r'''# sent_id = dev-s35
# text = Pomysł budowy bezpiecznego kąpieliska i ośrodka rekreacyjnego na terenie żwirowni nie jest nowy.
# orig_file_sentence 001#35
1	Pomysł	pomysł	NOUN	subst:sg:nom:m3	Animacy=Inan|Case=Nom|Gender=Masc|Number=Sing	13	nsubj	_	_
2	budowy	budowa	NOUN	subst:sg:gen:f	Case=Gen|Gender=Fem|Number=Sing	1	nmod	_	_
3	bezpiecznego	bezpieczny	ADJ	adj:sg:gen:n:pos	Case=Gen|Degree=Pos|Gender=Neut|Number=Sing	4	amod	_	_
4	kąpieliska	kąpielisko	NOUN	subst:sg:gen:n	Case=Gen|Gender=Neut|Number=Sing	2	nmod	_	_
5	i	i	CCONJ	conj	_	6	cc	_	_
6	ośrodka	ośrodek	NOUN	subst:sg:gen:m3	Animacy=Inan|Case=Gen|Gender=Masc|Number=Sing	4	conj	_	_
7	rekreacyjnego	rekreacyjny	ADJ	adj:sg:gen:m3:pos	Animacy=Inan|Case=Gen|Degree=Pos|Gender=Masc|Number=Sing	6	amod	_	_
8	na	na	ADP	prep:loc	AdpType=Prep|Case=Loc	9	case	_	_
9	terenie	teren	NOUN	subst:sg:loc:m3	Animacy=Inan|Case=Loc|Gender=Masc|Number=Sing	2	nmod	_	_
10	żwirowni	żwirownia	NOUN	subst:sg:gen:f	Case=Gen|Gender=Fem|Number=Sing	9	nmod	_	_
11	nie	nie	PART	qub	_	13	advmod	_	_
12	jest	być	AUX	fin:sg:ter:imperf	Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin	13	cop	_	_
13	nowy	nowy	ADJ	adj:sg:nom:m3:pos	Animacy=Inan|Case=Nom|Degree=Pos|Gender=Masc|Number=Sing	0	root	_	SpaceAfter=No
14	.	.	PUNCT	interp	_	13	punct	_	_'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)
        
if __name__ == '__main__':
    exec(r'''import os

while 'UDtrack' in os.listdir():
    os.chdir('UDtrack')

folders = [w for w in os.listdir() if LANGUAGE in w and '.' not in w]

rootfolder = os.getcwd()

answer = ''

for folder in folders:
    os.chdir(folder)
    files = os.listdir()
    if folder[3:] + '_THIS.conllu' in files:
        file = folder[3:] + '_THIS.conllu'
        # A final modified file should be have a name that ends with
        # _THIS.conllu
        # e.g. UD_Afrikaans-AfriBooms_THIS.conllu
    else:
        file = folder[3:] + '.conllu'
        # If such a file doesn't exist, just use the original which is named
        # folder without UD_ + '.conllu'

    answer += avaa(file)

    os.chdir(rootfolder)

answer = answer.strip()

g = answer.split('\n\n') # merkkijonot
''')
    g = [w for w in g if w.strip()]
    h = [to_ordered_dict(w) for w in g]
    zero = []
    for sent in h:
        if possessive(sent, dict)['zero_marked']:
            zero.append(sent)
    

