from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

possessive_pronouns = {'мой', 'твой', 'ее', 'его', 'наш', 'ваш', 'их', 'МОЙ', 'ТВОЙ', 'ЕЕ', 'ЕГО', 'НАШ', 'ВАШ', 'ИХ'}

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list, subtract=False):
    """The return type is a dict, the "returntype" argument is a needless remnant of a previous phase of this project.
It is included is to avoid crashing if an extra argument is given."""
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}


    sent_id = conllu_item['sent_id']

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i

    def find_chead_index(analysis):
        b = find_head_index(analysis)
        if analyses[b][6] == '0':
            return -1
        return find_head_index(analyses[b])

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]

    def has_adpositions(a): # True if there is an adposition referring to a
        for j in analyses:
            if j[3] == 'ADP' and j[7] == 'case' and j[6] == a[0]:
                return True
        return False
    
    head_exist = []

    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []

    for a in nanalyses:

        # possessive pronouns
        if a[2] in possessive_pronouns and a[3] == 'DET' and a[7] == 'det':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(a)

        # noun possessors
        if a[3] in {'NOUN', 'PROPN'} and 'Case=Gen' in a[5] and a[7] == 'nmod':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    if has_adpositions(a):
                        continue
                    # In Russian the letter 'ё' is usually replaced with 'е' though 'ё' has its place in the alphabet.
                    if a[1].lower().replace('ё', 'е') == a[2].lower().replace('ё', 'е'):
                        zero_marked.append(a)
                    else:
                        dep_marked.append(a)

    # almost copy-paste code.
    # Differences:
    # -canalyses instead of nanalyses
    # -a[7] replaced with firstconj[7]
    # -finding chead_index instead of head_index
    # -appending an edited version of the dependent
    # -no checking of adpositions (this has hopefully been done in the previous loop)

    for a in canalyses:

        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent

        # possessive pronouns
        if a[2] in possessive_pronouns and a[3] == 'DET' and firstconj[7] == 'det':
            head_i = find_chead_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    dep_marked.append(appendable(firstconj, a))

        # noun possessors
        if a[3] in {'NOUN', 'PROPN'} and 'Case=Gen' in a[5] and firstconj[7] == 'nmod':
            head_i = find_chead_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    if has_adpositions(a):
                        continue
                    # In Russian the letter 'ё' is usually replaced with 'е' though 'ё' has its place in the alphabet.
                    if a[1].lower().replace('ё', 'е') == a[2].lower().replace('ё', 'е'):
                        zero_marked.append(appendable(firstconj, a))
                    else:
                        dep_marked.append(appendable(firstconj, a))
    
    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer


test = r'''# sent_id = dev-s62
# text = Лопская (Мюгра, Большая, Пудос, Тюлле) -- река в России, протекает по территории Мурманской области и Карелии.
1	Лопская	ЛОПСКИЙ	ADJ	JJL	Animacy=Inan|Case=Nom|Gender=Fem|Number=Sing	12	nsubj	_	_
2	(	(	PUNCT	(	_	3	punct	_	SpaceAfter=No
3	Мюгра	МЮГРА	PROPN	NNP	Animacy=Inan|Case=Nom|Gender=Fem|Number=Sing	1	appos	_	SpaceAfter=No
4	,	,	PUNCT	,	_	5	punct	_	_
5	Большая	БОЛЬШОЙ	ADJ	JJL	Animacy=Inan|Case=Nom|Gender=Fem|Number=Sing	3	conj	_	SpaceAfter=No
6	,	,	PUNCT	,	_	7	punct	_	_
7	Пудос	ПУДОС	PROPN	NNP	Animacy=Inan|Case=Nom|Gender=Masc|Number=Sing	3	conj	_	SpaceAfter=No
8	,	,	PUNCT	,	_	9	punct	_	_
9	Тюлле	ТЮЛЛЕ	PROPN	NNP	Animacy=Inan|Case=Nom|Gender=Neut|Number=Sing	3	conj	_	SpaceAfter=No
10	)	)	PUNCT	)	_	3	punct	_	_
11	--	--	PUNCT	--	_	12	punct	_	_
12	река	РЕКА	NOUN	NN	Animacy=Inan|Case=Nom|Gender=Fem|Number=Sing	0	root	_	_
13	в	В	ADP	IN	_	14	case	_	_
14	России	РОССИЯ	PROPN	NNP	Animacy=Inan|Case=Loc|Gender=Fem|Number=Sing	12	nmod	_	SpaceAfter=No
15	,	,	PUNCT	,	_	16	punct	_	_
16	протекает	ПРОТЕКАТЬ	VERB	VBC	Aspect=Imp|Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin	12	conj	_	_
17	по	ПО	ADP	IN	_	18	case	_	_
18	территории	ТЕРРИТОРИЯ	NOUN	NN	Animacy=Inan|Case=Dat|Gender=Fem|Number=Sing	16	obl	_	_
19	Мурманской	МУРМАНСКИЙ	ADJ	JJL	Animacy=Inan|Case=Gen|Gender=Fem|Number=Sing	20	amod	_	_
20	области	ОБЛАСТЬ	NOUN	NN	Animacy=Inan|Case=Gen|Gender=Fem|Number=Sing	18	nmod	_	_
21	и	И	CCONJ	CC	_	22	cc	_	_
22	Карелии	КАРЕЛИЯ	PROPN	NNP	Animacy=Inan|Case=Gen|Gender=Fem|Number=Sing	20	conj	_	SpaceAfter=No
23	.	.	PUNCT	.	_	12	punct	_	_'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)
        
if __name__ == '__main__':
    exec(r'''import os

while 'UDtrack' in os.listdir():
    os.chdir('UDtrack')

folders = [w for w in os.listdir() if LANGUAGE in w and '.' not in w]

rootfolder = os.getcwd()

answer = ''

for folder in folders:
    os.chdir(folder)
    files = os.listdir()
    if folder[3:] + '_THIS.conllu' in files:
        file = folder[3:] + '_THIS.conllu'
        # A final modified file should be have a name that ends with
        # _THIS.conllu
        # e.g. UD_Afrikaans-AfriBooms_THIS.conllu
    else:
        file = folder[3:] + '.conllu'
        # If such a file doesn't exist, just use the original which is named
        # folder without UD_ + '.conllu'

    answer += avaa(file)

    os.chdir(rootfolder)

answer = answer.strip()

g = answer.split('\n\n') # merkkijonot
''')
    g = [w for w in g if w.strip()]
    h = [to_ordered_dict(w) for w in g]
    zero = []
    for sent in h:
        if possessive(sent, dict)['zero_marked']:
            zero.append(sent)
    

