from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

conj = 'conj'
punct = 'PUNCT'

def possessive(conllu_item, returntype=list, subtract=False):
    """The returntype argument is unused, the function returns a dict. It's a remnant of a previous phase of the project,
currently there only to avoid crashing if an extra argument is given."""
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}


    sent_id = conllu_item['sent_id']

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i

    def this_is_head_of(analysis):
        nonlocal analyses
        answer = []
        for a in analyses:
            if a[6] == analysis[0]:
                answer.append(a)
        return answer

    def find_chead_index(analysis):
        #if analysis[7] != conj: # only used for canalyses, this should be checked already
            #print(analysis, analysis[7])
        #assert analysis[7] == conj # only used for canalyses, this should be checked already
        b = find_head_index(analysis)
        if analyses[b][7] == '0':
            return -1
        return find_head_index(analyses[b])

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]
    

    #dep_head_exist = [] # items are dependents
    head_exist = [] # items are heads

    dep_marked = []
    head_marked = []
    double_marked = []
    zero_marked = []

    def genitive(analysis):
        return 'Case=Gen' in a[5] or 'Poss=Yes' in a[5] or 'GEN' in a[4].replace('-', '|').split('|')

    for a in nanalyses:
        # a is DEPENDENT
        head_i = find_head_index(a)
        if head_i >= 0: # head must exist

            #Genitive structures
            if (a[3] in {'NOUN', 'PROPN', 'PRON'} and a[7] == 'nmod:poss'):# or (a[3] == 'DET' and a[7] == 'det'):
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    a_is_genitive = genitive(a)
                    
                    if a_is_genitive:# and 'psor' not in analyses[head_i][5]: (there are no "psor"s in either Swedish file)
                        if a[3] in {'NOUN', 'PROPN'} and a[1].lower() == a[2].lower():
                            tiho = this_is_head_of(a)
                            if not tiho:
                                zero_marked.append(a)
                            elif all(b[7] == 'flat' and b[1].lower() == b[2].lower() for b in tiho):
                                zero_marked.append(a) # sent 4043 "New yorks" leads here, but it has a mistake in analysis,
                                # the lemma of "yorks" should be "york" not "yorks".
                            elif any(b[7] == 'flat' and b[1].lower() != b[2].lower() for b in tiho):
                                dep_marked.append(a)
                        else:
                            dep_marked.append(a)
                
                    elif 'Case=Gen' not in a[5]:# and 'psor' not in analyses[head_i][5]:
                        tiho = this_is_head_of(a)
                        for b in tiho:
                            #William Wilsons
                            if 'flat' in b[7] and 'Case=Gen' in b[5]:
                                dep_marked.append(a)
                                break
                            #släkt- och generationsbandens
                            elif '-|-' in a[4] and 'Case=Gen' in b[5]:
                                dep_marked.append(a)
                                break
                        else:
                            zero_marked.append(a)

            # av structures
            elif a[2] == 'av' and a[3] == 'ADP' and a[7] == 'case':
                part_index = int(a[0])
                dep_i = find_head_index(a)
                if dep_i >= 0:
                    dep = analyses[dep_i]
                    if dep[3] in {'NOUN', 'PROPN', 'PRON'} and dep[7] == 'nmod':
                        head_i = find_head_index(dep)
                        if head_i >= 0:
                            head = analyses[head_i]
                            if head[3] in {'NOUN', 'PROPN'}:
                                dep_marked.append(dep + [part_index] if subtract else dep)

                                for wd in canalyses.copy():
                                # find conjs for this "condition"
                                # Differences:
                                # dep -> wd
                                # firstconj_i
                                # replace dep(would be wd)[7] == 'nmod' with wd[6] == dep[0]
                                # append a manipulated version of the word (to_append) to calculate the dependency length correctly
                                # remove word from canalyses
                                    if wd[3] in {'NOUN', 'PROPN', 'PRON'} and wd[6] == dep[0]:
                                        firstconj_i = find_head_index(wd)
                                        head_i = find_head_index(analyses[firstconj_i])
                                        if head_i >= 0:
                                            head = analyses[head_i]
                                            if head[3] in {'NOUN', 'PROPN'}:
                                                head_position = head[0]
                                                to_append = wd[:6] + [head_position] + wd[7:]
                                                dep_marked.append(to_append + [part_index] if subtract else to_append)

                                                canalyses.remove(wd) # just for sure

    # almost copy-paste code.
    # Differences:
    # -canalyses instead of nanalyses
    # -dep[7] and a[7] replaced with firstconj[7]
    # -appending an edited version of the dependent
    # -skip adposition constructions
    # -this_is_head_of(to_append) -> [w for w in this_is_head_of(a) if w[7] != 'cc']


    for a in canalyses:

        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent

        head_i = find_head_index(a)
        if head_i >= 0: # head must exist

            #Genitive structures
            if (a[3] in {'NOUN', 'PROPN', 'PRON'} and firstconj[7] == 'nmod:poss'):# or (a[3] == 'DET' and firstconj[7] == 'det'):
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    a_is_genitive = genitive(a)

                    to_append = appendable(firstconj, a)
                    
                    if a_is_genitive:# and 'psor' not in analyses[head_i][5]: (there are no "psor"s in either Swedish file)
                        if a[3] in {'NOUN', 'PROPN'} and a[1].lower() == a[2].lower():
                            tiho = [w for w in this_is_head_of(a) if w[7] != 'cc'] # flats; not sure if this is correct
                            if not tiho:
                                zero_marked.append(to_append)
                            elif all(b[7] == 'flat' and b[1].lower() == b[2].lower() for b in tiho):
                                zero_marked.append(to_append) # sent 4043 "New yorks" leads here, but it has a mistake in analysis,
                                # the lemma of "yorks" should be "york" not "yorks".
                            elif any(b[7] == 'flat' and b[1].lower() != b[2].lower() for b in tiho):
                                dep_marked.append(to_append)
                        else:
                            dep_marked.append(to_append)
                
                    elif 'Case=Gen' not in a[5]:# and 'psor' not in analyses[head_i][5]:
                        tiho = [w for w in this_is_head_of(a) if w[7] != 'cc']
                        for b in tiho:
                            #William Wilsons
                            if 'flat' in b[7] and 'Case=Gen' in b[5]:
                                dep_marked.append(to_append)
                                break
                            #släkt- och generationsbandens
                            elif '-|-' in a[4] and 'Case=Gen' in b[5]:
                                dep_marked.append(to_append)
                                break
                        else:
                            zero_marked.append(to_append)

    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer

##test = r'''# sent_id = 2753
### text = Den aktuella Windows NT-användaren måste vara medlem i datorns administratörsgrupp.
##1	Den	den	DET	SG-DEF	Definite=Def|Gender=Com|Number=Sing|PronType=Art	4	det	_	_
##2	aktuella	aktuell	ADJ	POS-DEF	Case=Nom|Definite=Def|Degree=Pos|Number=Sing	4	amod	_	_
##3	Windows	Windows	PROPN	SG-NOM	Case=Nom	4	nmod	_	_
##4	NT-användaren	NT-användare	NOUN	SG-DEF-NOM	Case=Nom|Definite=Def|Gender=Com|Number=Sing	7	nsubj	_	_
##5	måste	måste	AUX	AUX	Mood=Ind|Tense=Pres|VerbForm=Fin|Voice=Act	7	aux	_	_
##6	vara	vara	AUX	INF-ACT	VerbForm=Inf|Voice=Act	7	cop	_	_
##7	medlem	medlem	NOUN	SG-IND-NOM	Case=Nom|Definite=Ind|Gender=Com|Number=Sing	0	root	_	_
##8	i	i	ADP	_	_	10	case	_	_
##9	datorns	dator	NOUN	SG-DEF-GEN	Case=Gen|Definite=Def|Gender=Com|Number=Sing	10	nmod:poss	_	_
##10	administratörsgrupp	administratörsgrupp	NOUN	SG-IND-NOM	Case=Nom|Definite=Ind|Gender=Com|Number=Sing	7	nmod	_	SpaceAfter=No
##11	.	.	PUNCT	Period	_	7	punct	_	_'''

test = r'''# sent_id = 3644
# text = Gengångaren på vinden ylade och bankade på rören så snart den kände att det höll på att bli för stillsamt, och små explosioner från Freds och Georges sovrum betraktades som någonting helt normalt.
1	Gengångaren	gengångare	NOUN	SG-DEF-NOM	Case=Nom|Definite=Def|Gender=Com|Number=Sing	4	nsubj	_	_
2	på	på	ADP	_	_	3	case	_	_
3	vinden	vind	NOUN	SG-DEF-NOM	Case=Nom|Definite=Def|Gender=Com|Number=Sing	1	nmod	_	_
4	ylade	yla	VERB	PAST-ACT	Mood=Ind|Tense=Past|VerbForm=Fin|Voice=Act	0	root	_	_
5	och	och	CCONJ	_	_	6	cc	_	_
6	bankade	banka	VERB	PAST-ACT	Mood=Ind|Tense=Past|VerbForm=Fin|Voice=Act	4	conj	_	_
7	på	på	ADP	_	_	8	case	_	_
8	rören	rör	NOUN	PL-DEF-NOM	Case=Nom|Definite=Def|Gender=Neut|Number=Plur	6	obl	_	_
9	så	så	ADV	_	_	10	advmod	_	_
10	snart	snart	ADV	_	Degree=Pos	12	mark	_	_
11	den	den	PRON	PERS-P3SG	Definite=Def|Gender=Com|Number=Sing|PronType=Prs	12	nsubj	_	_
12	kände	känna	VERB	PAST-ACT	Mood=Ind|Tense=Past|VerbForm=Fin|Voice=Act	4	advcl	_	_
13	att	att	SCONJ	_	_	18	mark	_	_
14	det	det	PRON	PERS-P3SG	Definite=Def|Gender=Neut|Number=Sing|PronType=Prs	18	nsubj	_	_
15	höll	hålla	AUX	PAST-AUX	Case=Nom	18	aux	_	_
16	på	på	ADP	_	_	15	compound:prt	_	_
17	att	att	PART	_	_	18	mark	_	_
18	bli	bli	AUX	INF-AUX	VerbForm=Inf|Voice=Act	12	ccomp	_	_
19	för	för	ADV	_	_	20	advmod	_	_
20	stillsamt	stillsam	ADJ	POS-SG-IND	Case=Nom|Definite=Ind|Degree=Pos|Number=Sing	18	xcomp	_	SpaceAfter=No
21	,	,	PUNCT	Comma	_	4	punct	_	_
22	och	och	CCONJ	_	_	28	cc	_	_
23	små	liten	ADJ	POS-PL-IND	Case=Nom|Definite=Ind|Degree=Pos|Number=Plur	24	amod	_	_
24	explosioner	explosion	NOUN	PL-IND-NOM	Case=Nom|Definite=Ind|Gender=Com|Number=Plur	30	nsubj:pass	_	_
25	från	från	ADP	_	_	29	case	_	_
26	Freds	Fred	PROPN	SG-GEN	Case=Gen	29	nmod:poss	_	_
27	och	och	CCONJ	_	_	28	cc	_	_
28	Georges	Georges	PROPN	SG-GEN	Case=Gen	26	conj	_	_
29	sovrum	sovrum	NOUN	IND-NOM	Case=Nom|Definite=Ind|Gender=Neut|Number=Sing	24	nmod	_	_
30	betraktades	betrakta	VERB	PAST-PASS	Mood=Ind|Tense=Past|VerbForm=Fin|Voice=Pass	4	conj	_	_
31	som	som	ADP	_	_	32	case	_	_
32	någonting	någonting	PRON	IND-SG	_	30	obl	_	_
33	helt	helt	ADV	_	Degree=Pos	34	advmod	_	_
34	normalt	normal	ADJ	POS-SG-IND	Case=Nom|Definite=Ind|Degree=Pos|Gender=Neut|Number=Sing	32	amod	_	SpaceAfter=No
35	.	.	PUNCT	Period	_	4	punct	_	_'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item])
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)
        

    

