from conllu_reader import *

if '\\' in __file__:
    slash = '\\'
else:
    slash = '/'

LANGUAGE = __file__.split(slash)[-1][:-3]

personal_pronouns = {'anh', 'ấy', 'chúng', 'chúng ta', 'chúng tôi', 'hắn',
                     'họ', 'mày', 'mình', 'nó', 'ông', 'ta', 'tôi', 'tui', 'y'}

punct = 'PUNCT'
conj = 'conj'

def possessive(conllu_item, returntype=list, subtract=False):
    """The returntype argument is unused, the function returns a dict. It's a remnant of a previous phase of the project,
currently there only to avoid crashing if an extra argument is given."""
    try:
        analyses = conllu_item['analyysit']
        nanalyses = [] #non-conj analyses
        canalyses = [] #conj analyses

        for i in analyses:
            if i[7] == conj:
                canalyses.append(i)
            else:
                nanalyses.append(i)
    except TypeError:
        print(conllu_item)
        return {'dep_marked':[], 'head_marked':[], 'double_marked':[], 'zero_marked':[], 'head_exist':[]}


    sent_id = conllu_item['sent_id']


    # Building dep_head_exist

    def find_head_index(analysis):
        head_str = analysis[6]
        head_i = int(head_str)-1
        return head_i

    dep_marked = []
    double_marked = []
    head_marked = []
    zero_marked = []
    head_exist = []

    for a in nanalyses: # Check each word in sentence if it's a dependent or an adposition (meaning: 'of') of a possessive NP.
##        if a[2] in personal_pronouns and a[3] == 'PROPN' and a[7] == 'det': # ORIGINAL PIECE OF CODE
##            head_i = find_head_index(a)
##            if head_i >= 0:
##                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
##                    for word in analyses:
##                        if word[2] == 'của' and word[6] == a[0] and word[3] == 'ADP' and word[7] == 'case':
##                            dep_marked.append(a)
##                            break
##                    else:
##                        zero_marked.append(a)
        

        if a[2] in personal_pronouns and a[3] == 'PROPN' and a[7] == 'det':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    for word in analyses:
                        if word[2] == 'của' and word[6] == a[0] and word[3] == 'ADP' and word[7] == 'case':
                            dep_marked.append(a)
                            break
                    else:
                        rhead_i = find_head_index(analyses[head_i]) # real head of the possessive NP
                        if rhead_i >= 0:
                            if analyses[rhead_i][3] in {'NOUN', 'PROPN'}:
                                zero_marked.append(a[:6] + [analyses[head_i][6]] + a[7:])


        elif a[2] == 'của' and a[3] == 'ADP' and a[7] == 'case': # This is the particle/adpositional case.
            part_index = int(a[0])
            dep_i = find_head_index(a)
            if dep_i > 0:
                dep = analyses[dep_i]
                if dep[3] in {'NOUN', 'PROPN'} and dep[7] == 'nmod':
                    head_i = find_head_index(dep)
                    if head_i >= 0:
                        if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                            dep_marked.append(dep + [part_index] if subtract else dep)
                            
                            for wd in canalyses.copy():
                            # find conjs for this "condition"
                            # Differences:
                            # assume dep is found:
                            # (start from testing the dependent, replacing dep with wd and also checking its head is dep (firstconj))
                            # dep -> wd
                            # firstconj_i
                            # replace dep(would be wd)[7] == 'nmod' with wd[6] == dep[0]
                            # append a manipulated version of the word (to_append) to calculate the dependency length correctly
                            # remove word from canalyses
                                if wd[3] in {'NOUN', 'PROPN'} and wd[6] == dep[0]:
                                    firstconj = dep
                                    head_i = find_head_index(firstconj)
                                    if head_i >= 0:
                                        head = analyses[head_i] # manual addition
                                        if head[3] in {'NOUN', 'PROPN'}: # originally: if analyses[head_i][3] in...
                                            head_position = head[0]
                                            to_append = wd[:6] + [head_position] + wd[7:]
                                            dep_marked.append(to_append + [part_index] if subtract else to_append)
                                            
                                            canalyses.remove(wd) # just for sure

    # almost copy-paste code.
    # Differences:
    # -firstconj lines
    # -canalyses instead of nanalyses
    # -a[7] replaced with firstconj[7]
    # -appending an edited version of the dependent
    # -skipping adpositions and others whose checking processes don't start with the dependent

    def appendable(firstconj, conj_a):
        return conj_a[:6] + [firstconj[6]] + conj_a[7:]

    for a in canalyses:
        firstconj_i = find_head_index(a)
        firstconj = analyses[firstconj_i]
        
        if firstconj_i < 0:
            continue # skip if conj refers to root
        if firstconj not in dep_marked+head_marked+double_marked+zero_marked:
            continue # skip if conj's head not any found possessive NP's dependent
        

        if a[2] in personal_pronouns and a[3] == 'PROPN' and firstconj[7] == 'det':
            head_i = find_head_index(a)
            if head_i >= 0:
                if analyses[head_i][3] in {'NOUN', 'PROPN'}:
                    for word in analyses:
                        if word[2] == 'của' and word[6] == a[0] and word[3] == 'ADP' and word[7] == 'case':
                            dep_marked.append(appendable(firstconj, a))
                            break
                    else:
                        rhead_i = find_head_index(analyses[head_i]) # real head of the possessive NP
                        if rhead_i >= 0:
                            if analyses[rhead_i][3] in {'NOUN', 'PROPN'}:
                                #Original line: zero_marked.append(a[:6] + [analyses[head_i][6]] + a[7:])
                                zero_marked.append(a[:6] + [firstconj[6]] + a[7:]) # Is this correct????



    answer = remove_duplicates({'dep_marked':dep_marked, 'head_marked':head_marked, 'double_marked':double_marked, 'zero_marked':zero_marked, 'head_exist':head_exist})

    for marking in answer: # copy-paste from tagchart_script. Adds number of PUNCTs between head and dependent as an int to the end of each dict value.
        # Replacing lause with conllu_item, kieli with LANGUAGE. Original lause['sent_id'] is now just sent_id.
        if marking == 'head_exist':
            for n, head in enumerate(answer[marking]):
                answer[marking][n].append(0)
        else:
            for n, dep in enumerate(answer[marking]):

                #lst[m:n] will return from m+1th element up to nth.
                #since we want from this+1st to head-1st, or head+1st to this-1st,
                #1 will be subtracted from the LARGER number in order to get the correct indices.
                
                
                this_i = int(dep[0])
                head_i = int(dep[6])
                # only those in between will be checked
                if this_i > head_i:
                    this_i -= 1
                elif this_i < head_i:
                    head_i -= 1
                else:
                    raise AssertionError("Head and dependent have the same ordinal: {} from {}".format(LANGUAGE, sent_id))
                if not (this_i >= 0 and head_i >= 0):
                    raise AssertionError("Head or dependent is root: {} from {}".format(LANGUAGE, sent_id))

                start = min(this_i, head_i)
                stop = max(this_i, head_i)

                answer[marking][n].append(len([w for w in conllu_item['analyysit'][start:stop] if w[3] == punct]))

    return answer

##test = r'''# sent_id = dev-s1
### text = Hay một người lính hải quân Pháp đã rải truyền đơn cho người dân nước Pháp cùng chống lại cuộc chiến phi nghĩa của quân đội Pháp tại VN.
##1	Hay	Hay	CCONJ	C	_	8	cc	_	_
##2	một	một	NUM	M	NumType=Card	4	nummod	_	_
##3	người	người	NOUN	Nc	_	4	compound	_	_
##4	lính	lính	NOUN	N	_	8	nsubj	_	_
##5	hải quân	hải quân	NOUN	N	_	4	compound	_	_
##6	Pháp	Pháp	NOUN	Np	_	5	compound	_	_
##7	đã	đã	X	R	_	8	advmod	_	_
##8	rải	rải	VERB	V	_	0	root	_	_
##9	truyền đơn	truyền đơn	NOUN	N	_	8	obj	_	_
##10	cho	cho	ADP	E	_	12	case	_	_
##11	người	người	NOUN	Nc	_	12	compound	_	_
##12	dân	dân	NOUN	N	_	8	obl	_	_
##13	nước	nước	NOUN	N	_	12	compound	_	_
##14	Pháp	Pháp	NOUN	Np	_	13	compound	_	_
##15	cùng	cùng	ADJ	A	_	8	xcomp	_	_
##16	chống	chống	VERB	V	_	15	xcomp	_	_
##17	lại	lại	X	R	_	16	advmod	_	_
##18	cuộc chiến	cuộc chiến	NOUN	N	_	16	obj	_	_
##19	phi nghĩa	phi nghĩa	ADJ	A	_	18	amod	_	_
##20	của	của	ADP	E	_	21	case	_	_
##21	quân đội	quân đội	NOUN	N	_	18	nmod	_	_
##22	Pháp	Pháp	NOUN	Np	_	21	compound	_	_
##23	tại	tại	ADP	E	_	24	case	_	_
##24	VN	VN	NOUN	Ny	_	21	nmod	_	SpaceAfter=No
##25	.	.	PUNCT	.	_	8	punct	_	_'''

#This sentence has manipulated tags.
#Should return zero marked ['8', 'ấy', 'ấy', 'PROPN', 'P', '_', '9', 'det', '_', '_']
test = r'''# sent_id = dev-s350
# text = biết cháu từng ở đó, ông ấy muốn hỏi cháu một số vấn đề".
1	biết	biết	VERB	V	_	0	root	_	_
2	cháu	cháu	NOUN	N	_	4	nsubj	_	_
3	từng	từng	X	R	_	4	advmod	_	_
4	ở	ở	VERB	V	_	1	obj	_	_
5	đó	đó	PROPN	P	_	4	obj	_	SpaceAfter=No
6	,	,	PUNCT	,	_	1	punct	_	_
7	ông	ông	NOUN	N	_	9	nmod	_	_
8	ấy	ấy	PROPN	P	_	7	det	_	_
9	muốn	muốn	NOUN	V	_	1	parataxis	_	_
10	hỏi	hỏi	VERB	V	_	9	obj	_	_
11	cháu	cháu	NOUN	N	_	10	obj	_	_
12	một	một	NUM	M	NumType=Card	13	nummod	_	_
13	số	số	NOUN	N	_	10	obj	_	_
14	vấn đề	vấn đề	NOUN	N	_	13	compound	_	SpaceAfter=No
15	"	"	PUNCT	"	_	1	punct	_	SpaceAfter=No
16	.	.	PUNCT	.	_	1	punct	_	_'''

conllu_item = to_ordered_dict(test)
d = possessive(conllu_item, dict)

if __name__ == '__main__':
    if type(d) == dict:
        for item in d:
            print(item, d[item])
    else:
        for item in d:
            print(item)


def str2poss(s, returntype=dict):
    conllu_item = to_ordered_dict(s)
    return possessive(conllu_item, returntype)

def str2bea(s, returntype=dict):
    d = str2poss(s, returntype)
    if type(d) == dict:
        for item in d:
            print(item, d[item], len(d[item]))
    else:
        for item in d:
            print(item)

def str2beastr(s):
    d = str2poss(s, dict)
    answer = []
    for item in d:
        answer.append('{} {} {}'.format(item, d[item], len(d[item])))
    return '\n'.join(answer)


def strip_indent(s, i=1):
    c = []
    for j in s.splitlines():
        c.append(j[4*i:])
    return '\n'.join(c)

